#!/usr/local/bin/perl

#
# ghplot.pl :  Prepare Genehunter output for displaying with gnuplot
# Syntax    :  perl gh_snp_npl.pl [-h] [-clean]
# Infiles   :  gh_<v>.out (default)
# Outfiles  :  myghnpl.gp, chr_line and NPL.plt
# Modified  : 11-Jan-2004	FR

use Cwd;

print "\n\n";
print " * * * * * * * * * * * GH_SNP_NPL.PL v1.0  * * * * * * * * * * * *\n";
print " * * * * *  Graph genome-wide NPL scores with GNUPLOT  * * * * * * \n";

##############################  V A R I A B L E S  #############################


my $maxlines = 999999; 	# Maximum number of lines (input-file)

my @xsums=();		# Positionen fr vertikale Chromosomen-Linie
my $xsum = 0.0;
my $xmin = 0;

my $ymax = 2;
my $ymin = -1;

my $pfad = getcwd();
my $datum = scalar localtime;
my %chrlabel=();
my $no_del =0;		# delete gnuplot infiles 0 - yes;  1 - no
my $hline=1;		# horizontal line: 1 - on
my $label =1;		# chromosome label 1 - on
my $mod=0;			# if 1 print NPLs of modscore analysis (should be the same)
my $score="Z-score";
my %Zscore=();		# determine which scoring function is used
my @scores=();

##################################   M A I N   #################################
foreach (@ARGV){
if ( /-h/i ) { &Help;}
if ( /-v/i ) { &Help;}
if ( /-nodelete/i ) { $no_del=1;}
if ( /-clean/i ) { &Clean;}
if ( /-nohline/i ) {$hline=0;}
if ( /-nolabel/i ) {$label=0;}
if ( /-mod/i ) {$mod=1;}
}

open (out4,">$pfad/NPL.plt") || die "could not open NPL.plt :$!\n";

foreach my $dir (c01,c02,c03,c04,c05,c06,c07,c08,c09,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23,cx,c24,cxy) {
	if (-d $dir) {
		$gh_tsh=0;	# is there a total stat het for this chrom
		foreach my $i ( 1..25) {
			$ghout="gh_$i\.out";	
			$in_file = "$dir/$ghout";
			if (-f "$in_file") {	  
				print "File: $in_file"; 
				&ReadNPL(); 
				print "\n";
			}
		} 
		if ($gh_tsh==1) {		# chromosome label
			my $xpos=(($lastxsum + $xsum)/2);
			my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
			if ($var eq "23"){$var = "X";}
			if ($var > 9){ $xpos -=30;}
			else { $xpos -=5;}
			$chrlabel{$var}=$xpos;
			$lastxsum=$xsum;
		}
		push (@xsums,$xsum);
		print out4 "\n";  
	}
	else { print "$dir not existing \n";
	}
} # end foreach

close (out4);

print "\n";
foreach $_ (keys %Zscore){
	print "score $_: $Zscore{$_}\n";
	push (@scores,$_);
}
if ($#scores > 0) { 
	print "Different score functions have been used!\n";
	$score="Z-score";
}
elsif ($#scores == 0) {$score = "Z-$scores[0]";}
else {$score="Z-all";}

&print_chr_line;
&pr_batch;

print "\nYou can now view results by \"gnuplot myghnpl.gp\" \n\n";
system ("gnuplot myghnpl.gp");

print "\nready ...\n\n";
if ($no_del == 0) { &Clean;}

#################################   R E A D N P L   #################################  

sub ReadNPL {
	my @v=();	# Array infile (Genehunter outfile)
	open (infile, "$in_file") || die "could not open $in_file :$!\n";
	while (<infile>){
		chomp;
		$_ =~ s/,-/, -/g;					# create equal number of columns 
		$_ =~ s/-INFINITY/-99.99/g;
		$_ =~ s/NaN/-99.99/g;				# Vorsicht , Genehunter hat hier nicht gerechnet
		push (@v,$_);
	}
	close (infile);

	my $jetzt = $maxlines;
	my $xmax = 0.0;
	my $count=0;
	my $c_tsh=0;	# Counter total stat het - lines

	foreach $line (@v) {
		@feld =	split(' ',$line);
		$count++;

		if (($feld[1] =~ /current/) && ($feld[2] =~ /score/)) {
			$score=$feld[6];
			$score=~ s/\'//g;
			$Zscore{$score}++;		# determine score function
		}
		
		if (($feld[3] =~ /het/) && ($feld[1] =~ /total/))   { 
			print " Read: total stat het";
			$c_tsh++;
			$gh_tsh=1;
		}
	
		if (($feld[2] =~ /alpha,/) && ($mod == 0) && ($c_tsh ==1))  { $jetzt = $count;}
		if (($feld[2] =~ /alpha,/) && ($mod == 1) && ($c_tsh ==2))  { $jetzt = $count;}

#		if ($feld[2] =~ /alpha,/)   { $jetzt = $count;}
		if ($feld[0] =~ /file/) { $jetzt = $maxlines;}
		if (($feld[5] =~ /npl_plot.ps/) || ($#feld < 1)) { $jetzt = $maxlines;}

		if ($count > $jetzt) {
			printf(out4 "%f %s\n", ($feld[0] + $xsum), $feld[4]);
			if ($feld[0] > $xmax) { $xmax = $feld[0];}  
			if ($feld[4] > $ymax) { $ymax = $feld[4];}
		}
	} # end foreach

	$xsum = $xsum + $xmax;
	if ($c_tsh > 1) { print "Warning:$in_file has more than one \"Total stat het\" region\n";}
	if ($c_tsh == 0) { print "Warning:$in_file has no \"Total stat het\" region\n";}
} ## end ReadNPL

######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {

#	$xsum = 3700;	# falls Bilder von verschiedenen Projekten die gleiche Achse haben sollen 
	$ymax += 1;						# y-achse auf ganze NPL-Zahl bringen
	@buff =	split('\.',$ymax);		# round to integer
	$ymax=$buff[0];
	
	open (out3,"> chr_line") || die "could not open chr_line :$!\n";

	if ($hline == 1){
		for (my $i=-2; $i<$ymax;$i++) {			# horizontale Linien bei NPLs = 1,2,3,4,..
			print out3 "0 $i\n$xsum $i\n\n";
		}
	}
	foreach (@xsums){						# vertikale Linien Chromosomenende 1,2,3,4,...22,x,xy
		print out3 "$_ $ymin \n$_ $ymax \n\n";         
	}		if (($feld[2] =~ /alpha,/) && ($mod == 0) && ($c_tsh ==1))  { $jetzt = $count;}
		if (($feld[2] =~ /alpha,/) && ($mod == 1) && ($c_tsh ==2))  { $jetzt = $count;}

	close (out3);
}
###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {

	if ($mod==0) {$titel = "$pfad - Genehunter NPL (Total stat)  - $datum";}
	else {$titel = "$pfad - Genehunter NPL (Total stat modscore)  - $datum";}
	
	open (out5,"> myghnpl.gp") || die "could not open myghnpl.gp :$!\n";

	print out5 "set autoscale\n";
	print out5 "set nokey\n";
	print out5 "set title \"$titel\\n\"\n" ;
	print out5 "set xlabel \"cM\"\n";
	print out5 "set ylabel \"$score\"\n";
	if ($label == 1){
		my $ypos = $ymax + (($ymax-$ymin)/20);		# 
		foreach (keys %chrlabel) {print out5 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	}
	print out5 "set size 1,0.6\n";
	print out5 "set terminal postscript color\n";
	if ($mod==0){print out5 "set output \"gh_snp_npl.ps\"\n";}
	else {print out5 "set output \"gh_snp_npl_mod.ps\"\n";}
	print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'NPL.plt\' with lines, \'chr_line\' with lines\n";
	print out5 "set terminal windows\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'NPL.plt\' with lines, \'chr_line\' with lines\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	close (out5);

} ## end pr_batch
					

#################################   H E L P   ##################################  
sub Help {

print " * * * * * (c) Created by Franz Rueschendorf 2-Jan-2003  * * * * * \n";
print " * * * * * * * * * Max-Delbrueck-Center, Berlin  * * * * * * * * * \n";
print " * * * * * * * * * email: fruesch\@mdc-berlin.de  * * * * * * * * * \n\n";
print "  Usage        : perl [path]gh_snp_npl.pl [-h] [-clean] [-nohline] [-nodelete] [-nolabel]\n";
print "  Infiles      : gh_<v>.out (default) \n";
print "  Outfiles     : myghnpl.gp, chr_line and NPL.plt \n" ;
print "  View results : gnuplot myghnpl.gp\n\n";
exit 0;
}

################################   C L E A N   #################################  

sub Clean { 
unlink glob("*.plt");
unlink glob("*.gp");
unlink glob("chr_line");
exit 0;
}

################################################################################
### end end gh_snp_npl.pl
